/*

DSock demo example for DSock, copyright (C) 2002 by DM&P.
This program provides you a fast way to write your application.
Just add you job as non-blocking function to main loop.

*/

#include "dsock_ap.h"
#include "my_job.h" /* Header file of your job */
#include <stdio.h>
#include <conio.h>

BOOL  InitServer(SOCKET *s, int nPort);
BOOL  UninitServer(SOCKET s);

int main()
{
  char    szBuf[32];
  BYTE    *pbyMac;
  DWORD   dwIp;
  SOCKET  sHttp;
  SOCKET  sFtp;
  SOCKET  sTelnet;
  SOCKET  sSmtp;

  printf("\nDM&P DSock Server Example\n\n");

  /* Initialize DSock library */
  if(DSock_Open() == FALSE)
  {
    printf("Unable to initialize socket library\n");
    return 1;
  }

  /* Use BOOTP/DHCP to get setup */
  if(DSock_DoBootp() == TRUE)
  {
    printf("Load network setup from BOOTP/DHCP\n");
  }
  else  /* Load setup from config file */
  {
    printf("Unable to load setup from BOOTP/DHCP server\n");
    DSock_LoadConfigFile("dsock.cfg");
    printf("Load network setup from DSOCK.CFG\n");
  }

  /* Dump network info */
  pbyMac = DSock_GetMacAddr();
  printf("MAC     : %02X-%02X-%02X-%02X-%02X-%02X\n", pbyMac[0], pbyMac[1],
           pbyMac[2], pbyMac[3], pbyMac[4], pbyMac[5]);

  dwIp = DSock_GetHostIp();
  inet_ntoa(szBuf, dwIp);
  printf("IP      : %s\n", szBuf);

  dwIp = DSock_GetNetmask();
  inet_ntoa(szBuf, dwIp);
  printf("Netmask : %s\n", szBuf);

  dwIp = DSock_GetGateway();
  inet_ntoa(szBuf, dwIp);
  printf("Gateway : %s\n", szBuf);

  dwIp = DSock_GetDomainNameServer();
  inet_ntoa(szBuf, dwIp);
  printf("DNS     : %s\n", szBuf);

  printf("HTTP service.....%s\n", InitServer(&sHttp, HTTP_PORT) ? "OK" : "Error");
  printf("FTP service......%s\n", InitServer(&sFtp, FTP_PORT) ? "OK" : "Error");
  printf("TELNET service...%s\n", InitServer(&sTelnet, TELNET_PORT) ? "OK" : "Error");
  printf("SMTP service.....%s\n", InitServer(&sSmtp, SMTP_PORT) ? "OK" : "Error");

  /*
  Main checking loop.
  You can add your job as non-blocking function and call it on the loop.
   */
  while(TRUE)
  {
    HttpServer(sHttp);
    FtpServer(sFtp);
    TelnetServer(sTelnet);
    SmtpServer(sSmtp);

    /*
    @CODE: Add you function here !!!
    Also, you can add your job on MyJob() in MY_JOB.C
    */
    if(MyJob() == 0)
      break;
  }

  UninitServer(sHttp);
  UninitServer(sFtp);
  UninitServer(sTelnet);

  /* Close DSock library */
  DSock_Close();

  return 0;
}

BOOL InitServer(SOCKET *s, int nPort)
{
  *s = SocketCreate(TCP_SOCKET);
  if(*s == INVALID_SOCKET)
    return FALSE;
  if(SocketBind(*s, 0L, nPort) == FALSE)
    return FALSE;
  if(SocketListen(*s) == FALSE)
    return FALSE;
  return TRUE;
}

BOOL UninitServer(SOCKET s)
{
  SocketDestory(s);
  return TRUE;
}
