/*

SMTP example for DSock, copyright (C) 2002 by DM&P.
This example will show you use Simple Mail Transfer Protocol to send mail.
The mail server and e-mail address for exmple should be replaced with yours.

*/

#include "..\dsock.h"
#include <stdio.h>
#include <stdlib.h>

#define SMTP_PORT 25

int   GetReply(SOCKET s); /* Get reply message from mail server */
BOOL  SendMail(SOCKET s); /* Send mail function */

int main()
{
  SOCKET  s;

  printf("\nDM&P DSock SMTP Example\n\n");

  /* Open DSock library */
  if(DSock_Open() == FALSE)
  {
    printf("Unable to initialize socket library\n");
    return 1;
  }

  /* Load network setup from config file */
  DSock_LoadConfigFile("dsock.cfg");

  s = SocketCreate(TCP_SOCKET);
  if(s == INVALID_SOCKET)
  {
    printf("SocketCreate() error\n");
    DSock_Close();
    return 1;
  }

  SendMail(s);

  SocketDestory(s);

  /* Close DSock library */
  DSock_Close();

  return 0;
}

int GetReply(SOCKET s)
{
  char  szTemp[1024];
  while(SocketDataReady(s) == 0);
  SocketGetString(s, szTemp, 1024);
  return atoi(szTemp);
}

int SendMail(SOCKET s)
{
  char  szBuf[32];
  char  szBuf2[32];
  BYTE  *pbyMac;
  DWORD dwIp;
  DWORD dwNetmask;
  DWORD dwGateway;
  DWORD dwDNS;

  printf("Connecting to mail server...");

  /* The mail server can be replaced with any server you want. */
  if(SocketConnect(s, inet_addr("210.242.175.7"), SMTP_PORT) == FALSE)
  {
    printf("SocketConnect() error\n");
    return FALSE;
  }

  printf("OK\n");

  SocketPutString(s, "HELO mail server\r\n");
  if(GetReply(s) < 0)
    return FALSE;

  SocketPutString(s, "MAIL FROM: test@demo.smtp.dsock\r\n");
  if(GetReply(s) < 0)
    return FALSE;

  SocketPutString(s, "RCPT TO: evanc@[210.242.175.7]\r\n");
  if(GetReply(s) < 0)
    return FALSE;

  SocketPutString(s, "DATA\r\n");
  while(TRUE)
  {
    int n = GetReply(s);
    if(n < 0)
      return 1;
    if(n == 354)
      break;
  }

  SocketPutString(s, "From: test@demo.dmp.dsock\r\n");
  SocketPutString(s, "To: evanc@[210.242.175.7]\r\n");
  SocketPutString(s, "Subject: DSock SMTP Example\r\n");
  SocketPutString(s, "\r\n");

  SocketPutString(s, "This is a test mail sent by DSock example program.\r\n");

  SocketPutString(s, "\r\n");
  SocketPutString(s, "** SMTP Client Network Information **\r\n");
  SocketPutString(s, "\r\n");

  /* Send network info */
  pbyMac = DSock_GetMacAddr();
  sprintf(szBuf, "MAC     : %02X-%02X-%02X-%02X-%02X-%02X\n", pbyMac[0],
            pbyMac[1], pbyMac[2], pbyMac[3], pbyMac[4], pbyMac[5]);
  SocketPutString(s, szBuf);

  dwIp = DSock_GetHostIp();
  inet_ntoa(szBuf2, dwIp);
  sprintf(szBuf, "IP      : %s\n", szBuf2);
  SocketPutString(s, szBuf);

  dwNetmask = DSock_GetNetmask();
  inet_ntoa(szBuf2, dwNetmask);
  sprintf(szBuf, "Netmask : %s\n", szBuf2);
  SocketPutString(s, szBuf);

  dwGateway = DSock_GetGateway();
  inet_ntoa(szBuf2, dwGateway);
  sprintf(szBuf, "Gateway : %s\n", szBuf2);
  SocketPutString(s, szBuf);

  dwDNS = DSock_GetDomainNameServer();
  inet_ntoa(szBuf2, dwDNS);
  sprintf(szBuf, "DNS     : %s\n", szBuf2);

  SocketPutString(s, szBuf);
  SocketPutString(s, "\r\n");

  SocketPutString(s, "\r\n.\r\n");
  if(GetReply(s) < 0)
    return FALSE;

  SocketPutString(s, "QUIT");

  SocketClose(s);
  printf("Mail is sent.\n");

  return TRUE;
}
