/*

TELNET server example for DSock, copyright (C) 2002 by DM&P.
Simple TELNET server demo and allow one user at a time. You can improve it to
support multi-user and support more commands when you need.

*/

#include "..\dsock.h"
#include <stdio.h>
#include <dos.h>
#include <time.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <dir.h>

/* TELNET Interpret As Command */
#define IAC   255
#define DONT  254
#define DO    253
#define WONT  252
#define WILL  251
#define SB    250
#define BREAK 241
#define SE    240

/* TELNET option */
#define TELOPT_ECHO     1
#define TELOPT_SGA      3
#define TELOPT_STATUS   5
#define TELOPT_TTYPE    24
#define NTELOPTS        24

#define BUF_SIZE        1024
#define TELNET_PORT     23
#define TELNET_TIME_OUT 30

BOOL  _bEcho = TRUE;
BOOL  _bSGA = FALSE;

BOOL  TelnetServer(SOCKET s);
void  SendIAC(SOCKET s, uchar cCmd, uchar cOpt);
void  RecvIAC(SOCKET s, uchar *pcCmd, uchar *pcOpt);
void  ReadCommand(SOCKET s, char *szBuf);

int main()
{
  DWORD   dwIp;
  char    szBuf[32];
  SOCKET  s;
  char    *sz = "-\\|/";
  uint    nCnt = 0;

  printf("\nDM&P DSock TELNET Server Example\n\n");

  /* Open DSock library */
  if(DSock_Open() == FALSE)
  {
    printf("Unable to initialize socket library\n");
    return 1;
  }

  /* Load network setup from config file */
  DSock_LoadConfigFile("dsock.cfg");

  /* Show server IP */
  dwIp = DSock_GetHostIp();
  inet_ntoa(szBuf, dwIp);
  printf("TELNET server IP : %s\n", szBuf);

  /* Ceate socket for FTP server */
  s = SocketCreate(TCP_SOCKET);
  if(s == INVALID_SOCKET)
  {
    printf("SocketCreate() error\n");
    DSock_Close();
    return 1;
  }

  if(SocketBind(s, 0L, TELNET_PORT) == FALSE)
  {
    printf("SocketBind() error\n");
    SocketDestory(s);
    DSock_Close();
    return 1;
  }

  if(SocketListen(s) == FALSE)
  {
    printf("SocketListen() error\n");
    SocketDestory(s);
    DSock_Close();
    return 1;
  }

  /*
  We change FtpServer() to non-blocking function for programmer to check TELNET
  connection. You can add HTTP or TELNET server checking routine into checking
  loop to support multiple services.
  */
  while(TRUE)
  {
    printf("%c\r", sz[nCnt++ / 500 % 4]);
    if(kbhit())         /* Press any key to break */
      break;
    TelnetServer(s);
  }

  /* Destory TELNET server socket */
  SocketDestory(s);

  /* Close DSock library */
  DSock_Close();

  return 0;
}

void SendIAC(SOCKET s, uchar cCmd, uchar cOpt)
{
  uchar ac[3];
  ac[0] = IAC;
  ac[1] = cCmd;
  ac[2] = cOpt;
  SocketSend(s, ac, 3);
}

void RecvIAC(SOCKET s, uchar *pcCmd, uchar *pcOpt)
{
  SocketGetChar(s, pcCmd);
  SocketGetChar(s, pcOpt);
}

void ReadCommand(SOCKET s, char *szBuf)
{
  uchar c;
  int   nIdx = 0;
  uchar cCmd, cOpt;
  while(TRUE)
  {
    if(SocketDataReady(s))
    {
      SocketGetChar(s, &c);
      if(c == IAC)
      {
        RecvIAC(s, &cCmd, &cOpt);
      }
      else if(c == 13)
      {
        SocketPutString(s, "\r\n");
        szBuf[nIdx] = 0;
        nIdx = 0;
        return;
      }
      else if(c == 0x7f)
      {
        if(nIdx)
        {
          SocketPutChar(s, 0x08);
          SocketPutChar(s, ' ');
          SocketPutChar(s, 0x08);
          szBuf[nIdx--] = 0;
        }
      }
      else
      {
        SocketPutChar(s, c);
        szBuf[nIdx++] = c;
      }
    }
  }
}

BOOL TelnetServer(s)
{
  DWORD          dwIp;
  char          szBuf[1024];
  static BOOL     bConnected = FALSE;
  static clock_t  clk;  /* For time out checking */

  /* Listen when socket is not connected */
  if(bConnected == FALSE)
  {
    if(SocketAccept(s, &dwIp) == FALSE)
      return TRUE;
    else
    {
      inet_ntoa(szBuf, dwIp);
      printf("TELNET: Accepting TELNET connecttion from %s\n", szBuf);
      SocketPutString(s, "DSock TELNET Server Example, service ready.\r\n");
      SendIAC(s, WILL, TELOPT_ECHO);

      /* Add your password checking here */
      SocketPutString(s, "Login: ");
      ReadCommand(s, szBuf);
      printf("TELNET: User \"%s\"\n", szBuf);
      SocketPutString(s, "Password: ");
      ReadCommand(s, szBuf);
      printf("TELNET: Password=\"%s\"\n", szBuf);

      SocketPutString(s, "\r\nWelcome to DSock TELNET Server Example.\r\n");
      SocketPutString(s, "Type \"help\" to get command set.\r\n\r\n");
      SocketPutString(s, "$");

      bConnected = TRUE;
      clk = clock();
    }
  }

  /* Socket already connected but ... */
  if(SocketIsConnected(s) == FALSE)
  {
    bConnected = FALSE;
    printf("TELNET: Remote disconected.\n");
    SocketClose(s);
    SocketBind(s, 0L, TELNET_PORT);
    SocketListen(s);
    return TRUE;
  }

  /* Is idle time out ? */
  if((clock() - clk) >= 18.2 * TELNET_TIME_OUT)
  {
    bConnected = FALSE;
    printf("TELNET: User idled %d seconds, disconnect.\n", TELNET_TIME_OUT);
    SocketClose(s);

    //SocketDestory(s);

    //s = SocketCreate(TCP_SOCKET);
    SocketBind(s, 0L, TELNET_PORT);
    SocketListen(s);
    return TRUE;
  }

  /* Is any data available ? */
  if(SocketDataReady(s) == FALSE)
    return TRUE;

  /* Reset timer when user send command */
  ReadCommand(s, szBuf);
  clk = clock();

  //SocketGetString(s,szBuf,1024);

  /* Debug information */
  printf("TELNET: %s\n", szBuf);

  /* Process command from user */
  if(stricmp("ver", szBuf) == 0)
  {
    SocketPutString(s, "DSock TELNET Server Demo\r\n");
  }
  else if(stricmp("help", szBuf) == 0)
  {
    SocketPutString(s, "help     show this page\r\n");
    SocketPutString(s, "ver      show porgram version\r\n");
    SocketPutString(s, "ls/dir   list files\r\n");
    SocketPutString(s, ".<cmd>   run <cmd> in DOS\r\n");
    SocketPutString(s, "logout   quit telnet server\r\n");
  }

  /* Codes copy from FTP server example */
  else if(stricmp("ls", szBuf) == 0 || stricmp("dir", szBuf) == 0)
  {
    struct ffblk  ffblk;
    int          nCode;
    char          *szAttr = "-rwxr--r--";
    static char   *szMonth[] =
    {
      "Jan",
      "Feb",
      "Mar",
      "Apr",
      "May",
      "Jun",
      "Jul",
      "Aug",
      "Sep",
      "Oct",
      "Nov",
      "Dec",
    };

    SocketPutString(s,
                        "150 Opening ASCII mode data connection for directory listing.\r\n");
    nCode = findfirst("*.*", &ffblk, 0xff);
    while(!nCode)
    {
      szAttr[0] = ffblk.ff_attrib & FA_DIREC ? 'd' : '-';
      SocketPutString(s, "%10s   1 501      501    %10lu %3s %2d  %4d %s\r\n",
                            szAttr, ffblk.ff_fsize,
                            szMonth[((ffblk.ff_fdate & 0x01e0) >> 5) - 1],
                            ffblk.ff_fdate & 0x001f, 1980 + (ffblk.ff_fdate >> 9),
                            ffblk.ff_name);
      nCode = findnext(&ffblk);
    }
  }

  /* Run DOS command */
  else if(szBuf[0] == '.')
  {
    char  *szTempFile = "dsock.tmp";
    FILE  *fp;
    sprintf(szBuf + 128, "%s > %s", szBuf + 1, szTempFile);
    system(szBuf + 128);
    fp = fopen(szTempFile, "rt");
    if(fp != NULL)
    {
      while(fgets(szBuf, 1024, fp) != NULL)
        SocketPutString(s, "%s\r", szBuf);
    }
  }
  else if(stricmp("logout", szBuf) == 0)
  {
    printf("TELNET: User logout.\n");
    SocketClose(s);
    bConnected = FALSE;
    SocketBind(s, 0L, TELNET_PORT);
    SocketListen(s);
    return TRUE;
  }
  else
  {
    SocketPutString(s, "Unknown command\r\n");
  }

  SocketPutString(s, "$");

  return TRUE;
}
